@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py,v 1.144 2006/07/19 20:46:28 richb Exp $"
-__version__   = "$Revision: 1.144 $"
-__date__      = "$Date: 2006/07/19 20:46:28 $"
+__id__        = "$Id: orca.py,v 1.154 2006/08/27 23:29:38 wwalker Exp $"
+__version__   = "$Revision: 1.154 $"
+__date__      = "$Date: 2006/08/27 23:29:38 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -49,10 +49,12 @@ import httpserver
 import keynames
 import keybindings
 import mag
+import orca_state
 import platform
 import rolenames
 import settings
 import speech
+
 import threading
 import util
 
@@ -67,28 +69,6 @@ from orca_i18n import _           # for gettext support
 #
 _userSettings = None
 
-# set each time a keyboard or braille event is received.
-#
-
-# The InputEvent instance representing the last input event.  This is
-# set each time a keyboard or braille event is received.
-#
-lastInputEvent = None
-
-# The last timestamp from a device event. Used to set focus for the Orca
-# configuration GUI.
-#
-lastTimestamp = None
-
-# The time that the last "No focus" event occured.
-#
-noFocusTimeStamp = 0.0
-
-# A new modifier to use (set by the press of any key in the
-# settings.orcaModifierKeys list) to represent the Orca modifier.
-#
-MODIFIER_ORCA = 8
-
 ########################################################################
 #                                                                      #
 # METHODS FOR HANDLING PRESENTATION MANAGERS                           #
@@ -149,10 +129,6 @@ def _switchToNextPresentationManager(script=None, inputEvent=None):
 #                                                                      #
 ########################################################################
 
-# The Accessible that has visual focus.
-#
-locusOfFocus = None
-
 def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     """Sets the locus of focus (i.e., the object with visual focus) and
     notifies the current presentation manager of the change.
@@ -163,29 +139,29 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
     - notifyPresentationManager: if True, propagate this event
     """
 
-    global locusOfFocus
-
-    if obj == locusOfFocus:
+    if obj == orca_state.locusOfFocus:
         return
 
-    oldLocusOfFocus = locusOfFocus
+    oldLocusOfFocus = orca_state.locusOfFocus
     if oldLocusOfFocus and not oldLocusOfFocus.valid:
         oldLocusOfFocus = None
 
-    locusOfFocus = obj
-    if locusOfFocus and not locusOfFocus.valid:
-        locusOfFocus = None
+    orca_state.locusOfFocus = obj
+    if orca_state.locusOfFocus and not orca_state.locusOfFocus.valid:
+        orca_state.locusOfFocus = None
 
-    if locusOfFocus:
+    if orca_state.locusOfFocus:
         appname = ""
-        if not locusOfFocus.app:
+        if not orca_state.locusOfFocus.app:
             appname = "None"
         else:
-            appname = "'" + locusOfFocus.app.name + "'"
+            appname = "'" + orca_state.locusOfFocus.app.name + "'"
 
         debug.println(debug.LEVEL_FINE,
                       "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
-                      % (appname, locusOfFocus.name, locusOfFocus.role))
+                      % (appname,
+                         orca_state.locusOfFocus.name,
+                         orca_state.locusOfFocus.role))
 
         if event:
             debug.println(debug.LEVEL_FINE,
@@ -203,7 +179,9 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
 
     if notifyPresentationManager and _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].\
-            locusOfFocusChanged(event, oldLocusOfFocus, locusOfFocus)
+            locusOfFocusChanged(event,
+                                oldLocusOfFocus,
+                                orca_state.locusOfFocus)
 
 def visualAppearanceChanged(event, obj):
     """Called (typically by scripts) when the visual appearance of an object
@@ -252,10 +230,9 @@ def _onMouseButton(e):
     Arguments:
     - e: at-spi event from the at-api registry
     """
-    global lastInputEvent
 
     event = atspi.Event(e)
-    lastInputEvent = MouseButtonEvent(event)
+    orca_state.lastInputEvent = MouseButtonEvent(event)
 
     # A mouse button event looks like: mouse:button:1p, where the
     # number is the button number and the 'p' is either 'p' or 'r',
@@ -435,44 +412,14 @@ def printActiveApp(script=None, inputEvent=None):
 
 def printAncestry(script=None, inputEvent=None):
     """Prints the ancestry for the current locusOfFocus"""
-    util.printAncestry(locusOfFocus)
+    util.printAncestry(orca_state.locusOfFocus)
     return True
 
 def printHierarchy(script=None, inputEvent=None):
     """Prints the application for the current locusOfFocus"""
-    if locusOfFocus:
-        util.printHierarchy(locusOfFocus.app, locusOfFocus)
-    return True
-
-########################################################################
-#                                                                      #
-# METHODS FOR HANDLING LEARN MODE.                                     #
-#                                                                      #
-########################################################################
-
-def enterLearnMode(script=None, inputEvent=None):
-    """Turns learn mode on.  The user must press the escape key to exit
-    learn mode.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    speech.speak(
-        _("Entering learn mode.  Press any key to hear its function. " \
-          + "To exit learn mode, press the escape key."))
-    braille.displayMessage(_("Learn mode.  Press escape to exit."))
-    settings.learnModeEnabled = True
-    return True
-
-def exitLearnMode(script=None, inputEvent=None):
-    """Turns learn mode off.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    speech.speak(_("Exiting learn mode."))
-    braille.displayMessage(_("Exiting learn mode."))
-    settings.learnModeEnabled = False
+    if orca_state.locusOfFocus:
+        util.printHierarchy(orca_state.locusOfFocus.app,
+                            orca_state.locusOfFocus)
     return True
 
 ########################################################################
@@ -586,10 +533,6 @@ def _isActionKey(event_string):
                   "orca._echoActionKey: returning: %s" % reply)
     return reply
 
-# Records the last time a key was echoed.
-#
-lastKeyEchoTime = None
-
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -602,12 +545,11 @@ def _keyEcho(event):
     - event: an AT-SPI DeviceEvent
     """
 
-    global lastKeyEchoTime
-
     # If this keyboard event was for an object like a password text
     # field, then don't echo it.
     #
-    if locusOfFocus and locusOfFocus.role == rolenames.ROLE_PASSWORD_TEXT:
+    if orca_state.locusOfFocus \
+        and (orca_state.locusOfFocus.role == rolenames.ROLE_PASSWORD_TEXT):
         return
 
     event_string = event.event_string
@@ -617,7 +559,7 @@ def _keyEcho(event):
     voices = settings.voices
     voice = voices[settings.DEFAULT_VOICE]
 
-    state = None
+    lockState = None
 
     # If key echo is enabled, then check to see what type of key event
     # it is and echo it via speech, if the user wants that type of key
@@ -643,9 +585,9 @@ def _keyEcho(event):
             modifiers = event.modifiers
             if event_string == _("Caps_Lock"):
                 if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    state = _(" off")
+                    lockState = _(" off")
                 else:
-                    state = _(" on")
+                    lockState = _(" on")
 
             elif event_string == _("Num_Lock"):
                 # [[[TODO: richb - we are not getting a correct modifier
@@ -677,8 +619,8 @@ def _keyEcho(event):
         if event_string in keynames.keynames:
             event_string = keynames.keynames[event_string]
 
-        if state:
-            event_string += state
+        if lockState:
+            event_string += lockState
 
         debug.println(debug.LEVEL_FINEST,
                       "orca._keyEcho: speaking: %s" % event_string)
@@ -686,7 +628,7 @@ def _keyEcho(event):
         # We keep track of the time as means to let others know that
         # we are probably echoing a key and should not be interrupted.
         #
-        lastKeyEchoTime = time.time()
+        orca_state.lastKeyEchoTime = time.time()
 
         speech.speak(event_string, voice)
 
@@ -703,11 +645,9 @@ def _processKeyboardEvent(event):
 
     Returns True if the event should be consumed.
     """
-    global lastInputEvent
-    global lastTimestamp
     global _orcaModifierPressed
 
-    lastTimestamp = event.timestamp
+    orca_state.lastInputEventTimestamp = event.timestamp
 
     # Log the keyboard event for future playback, if desired.
     # Note here that the key event_string being output is
@@ -749,7 +689,7 @@ def _processKeyboardEvent(event):
         _orcaModifierPressed = False
 
     if _orcaModifierPressed:
-        keyboardEvent.modifiers |= (1 << MODIFIER_ORCA)
+        keyboardEvent.modifiers |= (1 << settings.MODIFIER_ORCA)
 
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
@@ -773,15 +713,11 @@ def _processKeyboardEvent(event):
                 if event_string in keynames.keynames:
                     event_string = keynames.keynames[event_string]
                 speech.speak(event_string)
-            elif (keyboardEvent.type \
-                  == atspi.Accessibility.KEY_RELEASED_EVENT) \
-                 and (keyboardEvent.event_string == "Escape"):
-                exitLearnMode(None, keyboardEvent)
             consumed = True
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
-    lastInputEvent = keyboardEvent
+    orca_state.lastInputEvent = keyboardEvent
 
     return consumed or isOrcaModifier
 
@@ -799,7 +735,6 @@ def _processBrailleEvent(command):
 
     Returns True if the event was consumed; otherwise False
     """
-    global lastInputEvent
 
     # [[[TODO: WDW - probably should add braille bindings to this module.]]]
 
@@ -810,7 +745,7 @@ def _processBrailleEvent(command):
     speech.stop()
 
     event = BrailleEvent(command)
-    lastInputEvent = event
+    orca_state.lastInputEvent = event
 
     try:
         consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
@@ -970,6 +905,12 @@ def init(registry):
     if _initialized:
         return False
 
+    # Do not hang on initialization if we can help it.
+    #
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
+
     # Note that we have moved the Orca specific keybindings to the default
     # script, so _keyBindings is currently empty. The logic is retained
     # here, just in case we wish to reinstate them in the future.
@@ -990,6 +931,9 @@ def init(registry):
 
     registry.registerKeystrokeListeners(_processKeyboardEvent)
 
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
+
     _initialized = True
     return True
 
@@ -1002,6 +946,12 @@ def start(registry):
     if not _initialized:
         init(registry)
 
+    # Do not hang on startup if we can help it.
+    #
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
+
     try:
         speech.speak(_("Welcome to Orca."))
         braille.displayMessage(_("Welcome to Orca."))
@@ -1027,14 +977,19 @@ def start(registry):
 
     _switchToPresentationManager(0) # focus_tracking_presenter
 
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
+
     registry.start()
 
-def abort():
-    os._exit(1)
+def abort(exitCode=1):
+    os._exit(exitCode)
 
-def timeout():
-    print "TIMEOUT: Hung while trying to shutdown.  Aborting."
-    abort()
+def timeout(signum=None, frame=None):
+    debug.println(debug.LEVEL_SEVERE,
+                  "TIMEOUT: something has hung.  Aborting.")
+    debug.printStack(debug.LEVEL_ALL)
+    abort(50)
 
 def shutdown(script=None, inputEvent=None):
     """Exits Orca.  Unregisters any event listeners and cleans up.  Also
@@ -1049,12 +1004,11 @@ def shutdown(script=None, inputEvent=None):
     if not _initialized:
         return False
 
-    # [[[TODO: WDW - the timer stuff is an experiment to see if
-    # we can recover from hangs.  It's only experimental, so it's
-    # commented out for now.]]]
+    # Try to say goodbye, but be defensive if something has hung.
     #
-    #timer = threading.Timer(5.0, timeout)
-    #timer.start()
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
 
     speech.speak(_("goodbye."))
     braille.displayMessage(_("Goodbye."))
@@ -1081,8 +1035,8 @@ def shutdown(script=None, inputEvent=None):
 
     registry.stop()
 
-    #timer.cancel()
-    #del timer
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
 
     _initialized = False
     return True
@@ -1091,35 +1045,49 @@ exitCount = 0
 def shutdownOnSignal(signum, frame):
     global exitCount
 
-    #print "Shutting down and exiting due to signal =", signum
+    debug.println(debug.LEVEL_ALL,
+                  "Shutting down and exiting due to signal = %d" \
+                  % signum)
+
+    debug.println(debug.LEVEL_ALL, "Current stack is:")
+    debug.printStack(debug.LEVEL_ALL)
 
     # Well...we'll try to exit nicely, but if we keep getting called,
     # something bad is happening, so just quit.
     #
     if exitCount:
-        abort()
+        abort(signum)
     else:
         exitCount += 1
 
     # Try to do a graceful shutdown if we can.
     #
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.signal(signal.SIGALRM, settings.timeoutCallback)
+        signal.alarm(settings.timeoutTime)
+
     try:
         if _initialized:
             shutdown()
-            return
         else:
             # We always want to try to shutdown speech since the
             # speech servers are very persistent about living.
             #
             speech.shutdown()
             shutdown()
-            os._exit(0)
     except:
-        abort()
+        pass
+
+    if settings.timeoutCallback and (settings.timeoutTime > 0):
+        signal.alarm(0)
+
+    abort(signum)
 
 def abortOnSignal(signum, frame):
-    #print "Aborting due to signal =", signum
-    abort()
+    debug.println(debug.LEVEL_ALL,
+                  "Aborting due to signal = %d" \
+                  % signum)
+    abort(signum)
 
 def usage():
     """Prints out usage information."""
@@ -1130,6 +1098,7 @@ def usage():
     print _("-s, --setup, --gui-setup     Set up user preferences")
     print _("-t, --text-setup             Set up user preferences (text version)")
     print _("-n, --no-setup               Skip set up of user preferences")
+    #print _("-q, --quit                   Quits Orca (if shell script used)")
     print
     print _("If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used.")
     print
@@ -1137,6 +1106,18 @@ def usage():
     pass
 
 def main():
+    """The main entry point for Orca.  The exit codes for Orca will
+    loosely be based on signals, where the exit code will be the
+    signal used to terminate Orca (if a signal was used).  Otherwise,
+    an exit code of 0 means normal completion and an exit code of 50
+    means Orca exited because of a hang."""
+
+    # Method to call when we think something might be hung.
+    #
+    settings.timeoutCallback = timeout
+
+    # Various signal handlers we want to listen for.
+    #
     signal.signal(signal.SIGHUP, shutdownOnSignal)
     signal.signal(signal.SIGINT, shutdownOnSignal)
     signal.signal(signal.SIGTERM, shutdownOnSignal)
