@@ -24,9 +24,9 @@
 
 """Displays a GUI for the user to set Orca preferences."""
 
-__id__        = "$Id: orca_gui_prefs.py,v 1.50 2006/08/08 15:04:40 richb Exp $"
-__version__   = "$Revision: 1.50 $"
-__date__      = "$Date: 2006/08/08 15:04:40 $"
+__id__        = "$Id: orca_gui_prefs.py,v 1.51 2006/08/28 17:00:57 richb Exp $"
+__version__   = "$Revision: 1.51 $"
+__date__      = "$Date: 2006/08/28 17:00:57 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -158,7 +158,6 @@ class orcaSetupGUI(GladeWrapper):
         self.serverChoices = None
         self._setupServers()
         self._setupVoices()
-        self.prefsDict["enableSpeech"] = True
         self._initGUIState()
         self.initializing = False
 
@@ -1386,7 +1385,8 @@ class orcaSetupGUI(GladeWrapper):
         - widget: the component that generated the signal.
         """
 
-        self.prefsDict["enableSpeech"] = True
+        enable = self.speechSupportCheckbutton.get_active()
+        self.prefsDict["enableSpeech"] = enable
         self.prefsDict["speechServerFactory"] = self.factory
         self.prefsDict["speechServerInfo"] = self.server
         self.prefsDict["voices"] = {
