@@ -19,16 +19,15 @@
 
 """Custom script for Mozilla.  NOT WORKING WELL AT THE MOMENT."""
 
-__id__        = "$Id: Mozilla.py,v 1.11 2006/06/10 00:54:48 wwalker Exp $"
-__version__   = "$Revision: 1.11 $"
-__date__      = "$Date: 2006/06/10 00:54:48 $"
+__id__        = "$Id: Mozilla.py,v 1.13 2006/07/28 17:33:18 wwalker Exp $"
+__version__   = "$Revision: 1.13 $"
+__date__      = "$Date: 2006/07/28 17:33:18 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import orca.atspi as atspi
 import orca.debug as debug
 import orca.default as default
-import orca.orca as orca
 import orca.rolenames as rolenames
 import orca.settings as settings
 import orca.speech as speech
@@ -147,7 +146,7 @@ class Script(default.Script):
         #print "Mozilla.onCaretMoved"
         #print "  source        =", event.source
         #print "  source.parent =", event.source.parent
-        #print "  lof           =", orca.locusOfFocus
+        #print "  lof           =", state.locusOfFocus
         
         # We always automatically go back to focus tracking mode when
         # the caret moves in the focused object.
@@ -193,10 +192,14 @@ class Script(default.Script):
         # get focus immediately after the menu gets focus.  So...we
         # compress the events.
         #
-        if (event.source.role == rolenames.ROLE_MENU) \
-           and event.source.parent \
-           and (event.source.parent.role == rolenames.ROLE_MENU_BAR):
-            return
+        # [[[WDW - commented this out on 27-Jul-2006 based upon feedback
+        # from Lynn Monsanto that it was getting in the way for Firefox
+        # and Yelp.]]]
+        #
+        #if (event.source.role == rolenames.ROLE_MENU) \
+        #   and event.source.parent \
+        #   and (event.source.parent.role == rolenames.ROLE_MENU_BAR):
+        #    return
         
         default.Script.onFocus(self, event)
 
